package edu.uwyo.crawl.contentMatcher;

/**
 * The central class for representing the tokens of a regular expression.
 * All found tokens should be placed in this object.
 * To convert the regular expression back into its original string, call {@link #toString()}.
 * The minimum number of characters required to match the expression
 *  can be obtained via {@link #length()}.
 * @version 6 March, 2019
 * @author Christopher Stanton
 */
public class ExpressionTokenizer extends AbstractExpression {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link ExpressionTokenizer}. */
    protected ExpressionTokenizer() {
        super();
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void toString(StringBuilder builder) {
        for (int i = 0; i < expressions.size(); i++) {
            for (AbstractExpression expression : expressions.get(i)) {
                expression.toString(builder);
            }
            if (i < expressions.size() - 1) {
                builder.append("|");
            }
        }
    }

    @Override
    protected void add(AbstractExpression expression, int length) {
        expressions.get(currOrGroup).add(expression);
        this.length.set(currOrGroup, this.length.get(currOrGroup) + length);
    }

    /** Does nothing. */
    @Override
    protected void zeroOrMore() {}

    /** Does nothing. */
    @Override
    protected void oneOrMore() {}

    /** Does nothing. */
    @Override
    protected void repeat(int min, int max) {}

    /**
     * Converts this entire expression into its written, human-readable form
     *  by calling {@link #toString(StringBuilder)}.
     */
    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        toString(builder);
        return builder.toString();
    }
}
