package edu.uwyo.crawl.contentMatcher;

/**
 * An interface defining the parameters that can be tuned for content-based file searches.
 * @version 19 March, 2019
 * @author Christopher Stanton
 */
public interface TunableContentSearch {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Sets the parameters used when conducting the search.
     * <p>
     * The {@code bufferSize} determines how many bytes are read at a time for each file.
     * This value should be closely related to the processor cache size to avoid cache misses
     *  or wasted reads.
     * However when searching network files, in which the network latency is much larger
     *  than the latency induced by a cache miss, the buffer size should be set much higher
     *  (typically on the order of megabytes).
     * <p>
     * The {@code maxChunkBytes} determines at which point a file is "too large" to fit in memory.
     * Files exceeding this size will be split to minimize memory usage.
     * Because matches cannot be made across the split, this value should typically be
     *  on the order of hundreds of megabytes to reduce false negatives over the split point.
     * @param bufferSize the integer size of the buffer for reading in each file, in bytes
     * @param maxChunkBytes the integer maximum size above which files are split, in bytes
     */
    public void tuneParameters(int bufferSize, int maxChunkBytes);
}
