package edu.uwyo.crawl.database;

import java.io.IOException;

/**
 * An interface to be implemented by serializable objects.
 * In addition to the methods within this interface, implementing classes should typically have:
 * <ul>
 *  <li>A public or private, no-argument constructor.
 *  Will be used for creating a new instance via {@link #deserialize(DataInput)}.</li>
 *  <li>A public constant {@code INSTANCE}, containing an "empty" version of the object.
 *  Will be used to drive the serialization process.</li>
 * </ul>
 * @version 6 March, 2019
 * @author Christopher Stanton
 */
public interface Serializer<T> {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Serializes this object to the given output stream.
     * @param output the {@link DataOutput} stream to which this object should be written
     * @param instance the object to be written
     * @throws IOException if an I/O error occurs
     */
    public void serialize(DataOutput output, T instance) throws IOException;

    /**
     * Deserializes this object from the given input stream.
     * @param input the {@link DataInput} stream from which this object should be read
     * @return the object that was read
     * @throws IOException if an I/O error occurs
     */
    public T deserialize(DataInput input) throws IOException;
}
