package edu.uwyo.crawl.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import edu.uwyo.Main;
import net.javamon.dev.Ext;
import net.javamon.dev.Prompt;
import net.javamon.dev.Resources;
import net.javamon.dev.utils.RelativeFileChooser;

/**
 * A listener for saving the current results to a file.
 * @version 6 March, 2019
 * @author Christopher Stanton
 */
public class SaveListener implements ActionListener {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private final FrameMain parent;
    private final RelativeFileChooser fileChooser;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link SaveListener}. */
    protected SaveListener(FrameMain parent) {
        this.parent = parent;
        fileChooser = RelativeFileChooser.build("mainChooseSnapshotH", Ext.FIND, false);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void actionPerformed(ActionEvent e) {
        File snapshot = null;
        if (Main.currSnapshot() != null) {
            fileChooser.setCurrentDirectory(Main.currSnapshot());
        }

        int choice;
        while (true) {
            snapshot = null;
            choice = fileChooser.showSaveDialog(parent.main().panel);

            if (choice == JFileChooser.APPROVE_OPTION) {
                snapshot = fileChooser.getSelectedFile();
                String path = snapshot.getAbsolutePath();
                if (!path.endsWith(Ext.FIND)) {
                    snapshot = new File(path + Ext.FIND);
                }

                if (snapshot.exists()) {
                    choice = Prompt.show(Resources.__("mainResultsSaveConfirmH"),
                            Resources.__("mainResultsSaveConfirmMsg", snapshot.getAbsolutePath()));
                    if (choice == JOptionPane.YES_OPTION) {
                        Main.saveSnapshot(snapshot);
                        break;
                    }
                } else {
                    Main.saveSnapshot(snapshot);
                    break;
                }
            } else {
                break;
            }
        }
    }
}
