package edu.uwyo.crawl.ui.frameAbout;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.KeyStroke;

import edu.uwyo.crawl.ui.FrameMain;
import edu.uwyo.crawl.ui.frameAbout.panels.PanelMain;
import net.javamon.dev.Resources;

/**
 * The frame used for displaying instructional information to the user.
 * @version 15 February, 2019
 * @author Christopher Stanton
 */
public class FrameAbout implements WindowListener {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private PanelMain panelMain;
    private JFrame frame;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link FrameAbout}. */
    @SuppressWarnings("serial")
    public FrameAbout(FrameMain parent) {
        frame = new JFrame(Resources.__("mainAboutTitleNoHTML"));
        frame.setMinimumSize(new Dimension(600, 560));
        frame.setLayout(new GridLayout(0, 1));
        frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        frame.addWindowListener(this);

        panelMain = new PanelMain();
        frame.add(panelMain.panel);

        frame.pack();
        frame.setLocationRelativeTo(parent.main().panel);

        frame.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
                .put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "closeWindow");
        frame.getRootPane().getActionMap().put("closeWindow", new AbstractAction() {
            @Override
            public void actionPerformed(ActionEvent e) {
                hide();
            }
        });
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Does nothing by default. */
    @Override
    public void windowIconified(WindowEvent e) {}

    /** Does nothing by default. */
    @Override
    public void windowDeiconified(WindowEvent e) {}

    /** Does nothing by default. */
    @Override
    public void windowActivated(WindowEvent e) {}

    /** Does nothing by default. */
    @Override
    public void windowDeactivated(WindowEvent e) {}

    @Override
    public void windowClosing(WindowEvent e) {
        hide();
    }

    /** Does nothing by default. */
    @Override
    public void windowOpened(WindowEvent e) {}

    /** Does nothing by default. */
    @Override
    public void windowClosed(WindowEvent e) {}

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Shows this frame. */
    public void show() {
        frame.setVisible(true);
        panelMain.validate();
    }

    /** Hides this frame. */
    public void hide() {
        frame.setVisible(false);
    }

    /** @return this frame's {@link PanelMain} */
    public PanelMain main() {
        return panelMain;
    }
}
