package edu.uwyo.crawl.ui.frameExclude.panels;

import javax.swing.JButton;

import edu.uwyo.Main;
import edu.uwyo.crawl.RestorableView;
import net.javamon.dev.utils.JScrollPaneText;
import net.javamon.dev.utils.layout.AbstractPanel;
import net.javamon.dev.utils.layout.ValidateFocusListener;
import net.javamon.dev.utils.layout.ValidateKeyListener;
import net.javamon.dev.utils.textPane.JTextPaneMarkup;

/**
 * The panel indicating which file types should be excluded from searches.
 * @version 15 February, 2019
 * @author Christopher Stanton
 */
public class PanelExtensions extends AbstractPanel implements RestorableView {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                   CONSTANTS                     ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private static final String DELIMITERS = "[ \t\n,]+";

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public JScrollPaneText fileTypes;
    public JButton excludeNonText;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link PanelExtensions}. */
    public PanelExtensions() {
        super("mainExcludeExtensionsBdr", 200, 125);
        setExclude();
        setTypes();
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void setEnabled(boolean enabled) {
        excludeNonText.setEnabled(enabled);
        fileTypes.setEnabled(enabled);
    }

    @Override
    public void validate() {
        String text = fileTypes.getText().trim();
        if (text.length() == 0) {
            fileTypes.setText("");
            return;
        }

        String[] types = text.split(DELIMITERS);
        String[] toCheck = new String[types.length];

        StringBuilder validated = new StringBuilder();
        for (int i = 0; i < types.length; i++) {
            String validatedType = "." + types[i].replace(".", "");
            toCheck[i] = validatedType;
            validated.append(validatedType + " ");
        }
        fileTypes.setText(validated.toString().trim());
    }

    @Override
    public void restoreView() {
        String[] ext = Main.excludeList().getExt();
        if (ext == null) {
            fileTypes.setText("");
        } else {
            StringBuilder builder = new StringBuilder();
            for (int i = 0; i < ext.length; i++ ) {
                builder.append(ext[i] + " ");
            }
            fileTypes.setText(builder.toString());
        }
    }

    @Override
    public void saveView() {
        Main.excludeList().setExt(getExcludedTypes());
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** @return an array of String excluded file types, or {@code null} if all types are allowed */
    public String[] getExcludedTypes() {
        String text = fileTypes.getText();
        if (text.length() == 0) {
            return null;
        }
        return text.split(DELIMITERS);
    }

    /** Sets the exclude non-text button. */
    private void setExclude() {
        excludeNonText = createButton("mainExcludeNonText", "mainExcludeNonTextTT", -1, -1);
        excludeNonText.addActionListener(new ExcludeNonTextListener(this));
        layout(EAST, excludeNonText, -5, EAST, panel);
        layout(NORTH, excludeNonText, 5, NORTH, panel);
        add(excludeNonText);
    }

    /** Sets the types field. */
    private void setTypes() {
        fileTypes = createScrollText(JTextPaneMarkup.MARKUP_NONE, null,
                "mainExcludeExtensionsTT", 4, 5, true, true, false, true,
                borderWidth() - left() - right(),
                borderHeight() - top() - bottom());
        fileTypes.getPane().addFocusListener(new ValidateFocusListener(this));
        fileTypes.getPane().addKeyListener(new ValidateKeyListener(this));
        layoutFill(fileTypes, 5, excludeNonText, panel, panel, panel);
        add(fileTypes);
    }
}
