package edu.uwyo.crawl.ui.frameExclude.panels;

import java.io.File;
import java.io.IOException;

import edu.uwyo.Main;
import edu.uwyo.crawl.RestorableView;
import net.javamon.dev.utils.JScrollPaneText;
import net.javamon.dev.utils.layout.AbstractPanel;
import net.javamon.dev.utils.layout.ValidateFocusListener;
import net.javamon.dev.utils.textPane.JTextPaneMarkup;

/**
 * The panel indicating which files should be excluded from searches.
 * @version 15 February, 2019
 * @author Christopher Stanton
 */
public class PanelFiles extends AbstractPanel implements RestorableView {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                   CONSTANTS                     ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private static final String DELIMITERS = "[\n]+";

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public JScrollPaneText files;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link PanelFiles}. */
    public PanelFiles() {
        super("mainExcludeFilesBdr", 200, 100);
        setFiles();
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void setEnabled(boolean enabled) {
        files.setEnabled(enabled);
    }

    @Override
    public void validate() {
        String text = files.getText();
        if (text.length() == 0) { return; }

        String[] files = text.split(DELIMITERS);
        String[] toCheck = new String[files.length];

        StringBuilder validated = new StringBuilder();
        for (int i = 0; i < files.length; i++) {
            try {
                String validatedType = new File(files[i]).getCanonicalPath();
                toCheck[i] = validatedType;
                validated.append(validatedType + "\n");
            } catch (IOException e) {}
        }
        this.files.setText(validated.toString());
    }

    @Override
    public void restoreView() {
        String[] files = Main.excludeList().getFiles();
        if (files == null) {
            this.files.setText("");
        } else {
            StringBuilder builder = new StringBuilder();
            for (int i = 0; i < files.length; i++ ) {
                builder.append(files[i] + "\n");
            }
            this.files.setText(builder.toString());
        }
    }

    @Override
    public void saveView() {
        Main.excludeList().setFiles(getExcludedFiles());
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** @return an array of String excluded files, or {@code null} if no files are excluded */
    public String[] getExcludedFiles() {
        String text = files.getText();
        if (text.length() == 0) {
            return null;
        }
        return text.split(DELIMITERS);
    }

    /** Sets the files field. */
    private void setFiles() {
        files = createScrollText(JTextPaneMarkup.MARKUP_NONE, null,
                "mainExcludeFilesTT", 4, 5, true, false, false, true,
                borderWidth() - left() - right(),
                borderHeight() - top() - bottom());
        files.getPane().addFocusListener(new ValidateFocusListener(this));
        layoutFill(files, 5, panel, panel, panel, panel);
        add(files);
    }
}
