package edu.uwyo.crawl.ui.frameExclude.panels;

import java.awt.Color;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import javax.swing.JTextField;

import edu.uwyo.Main;
import edu.uwyo.crawl.RestorableView;
import net.javamon.dev.utils.JLabelHTML;
import net.javamon.dev.utils.layout.AbstractPanel;
import net.javamon.dev.utils.layout.ValidateFocusListener;
import net.javamon.dev.utils.layout.ValidateKeyListener;

/**
 * The panel indicating which file name patterns should be excluded from searches.
 * @version 15 February, 2019
 * @author Christopher Stanton
 */
public class PanelRegex extends AbstractPanel implements RestorableView {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                   CONSTANTS                     ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private static final Color BAD_PATTERN = new Color(255, 200, 200);
    private static final Color GOOD_PATTERN = new JTextField("").getBackground();

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public JTextField regex;
    public JLabelHTML badPattern;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link PanelRegex}. */
    public PanelRegex() {
        super("mainExcludeRegexBdr", 200, 75);

        setRegex();
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void setEnabled(boolean enabled) {
        regex.setEnabled(enabled);
    }

    @Override
    public void validate() {
        regex.setBackground(GOOD_PATTERN);
        badPattern.setVisible(false);

        if (regex.isEnabled() && regex.isEditable()) {
            try {
                Pattern.compile(regex.getText());
            } catch (PatternSyntaxException e) {
                regex.setBackground(BAD_PATTERN);
                if (badPattern != null) {
                    badPattern.setVisible(true);
                }
            }
        } else {
            regex.setText("");
        }

        if (regex.getText().trim().length() > 0 && !badPattern.isVisible()) {
            Main.excludeList().setPattern(Pattern.compile(regex.getText()));
        } else {
            Main.excludeList().setPattern(null);
        }
    }

    @Override
    public void restoreView() {
        String pattern = Main.excludeList().getPattern();
        if (pattern != null) {
            regex.setText(pattern);
        } else {
            regex.setText("");
        }
    }

    @Override
    public void saveView() {
        Main.excludeList().setPattern(null);
        try {
            if (regex.getText().trim().length() > 0 && !badPattern.isVisible()) {
                Main.excludeList().setPattern(Pattern.compile(regex.getText()));
            }
        } catch (PatternSyntaxException e) {}
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** @return the {@link Pattern} of file names to be excluded */
    public Pattern getRegex() {
        return (regex.getText().length() == 0) ? null : Pattern.compile(regex.getText());
    }

    /** Sets the regex field. */
    private void setRegex() {
        regex = createTextField(null, 25, "mainExcludeRegexTT");
        regex.addFocusListener(new ValidateFocusListener(this));
        regex.addKeyListener(new ValidateKeyListener(this));
        layout(WEST, regex, 5, WEST, panel);
        layout(NORTH, regex, 5, NORTH, panel);
        layout(EAST, regex, -5, EAST, panel);
        add(regex);

        badPattern = createLabel("mainPanelOptionsPatternBad", null);
        badPattern.setForeground(new Color(255, 0, 0));
        layout(EAST, badPattern, -5, EAST, panel);
        layout(NORTH, badPattern, 5, SOUTH, regex);
        add(badPattern);
    }
}
