package edu.uwyo.crawl.ui.frameFlags.panels;

import javax.swing.JCheckBox;

import edu.uwyo.Main;
import edu.uwyo.crawl.RestorableView;
import edu.uwyo.crawl.ui.FrameMain;
import edu.uwyo.crawl.ui.panels.PanelFiles;
import net.javamon.dev.Resources;
import net.javamon.dev.utils.HTML;
import net.javamon.dev.utils.JLabelHTML;
import net.javamon.dev.utils.PanelColor;
import net.javamon.dev.utils.layout.AbstractPanel;

/**
 * A panel representing a single status flag filter.
 * @version 15 February, 2019
 * @author Christopher Stanton
 */
public class PanelFlag extends AbstractPanel implements RestorableView {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public JCheckBox enabled;
    public JLabelHTML label;

    private PanelColor color;
    private final int flag;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Creates a new {@link PanelFlag}.
     * @param parent the {@link FrameMain} to use for triggering updates to search results,
     *  or {@code null} if none
     * @param flag the integer index of the flag represented by this panel
     */
    public PanelFlag(FrameMain parent, int flag) {
        super(300, 25);
        this.flag = flag;

        setFlag(parent);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void setEnabled(boolean enabled) {}

    @Override
    public void validate() {}

    @Override
    public void restoreView() {
        String lastFlag = Main.readSetting("lastFlagFilter" + flag);
        if (lastFlag != null) {
            enabled.setSelected(Boolean.parseBoolean(lastFlag));
        }
    }

    @Override
    public void saveView() {
        Main.writeSetting("lastFlagFilter" + flag, enabled.isSelected() ? "True" : "False");
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Sets the flag chooser and color panel. */
    private void setFlag(FrameMain parent) {
        final String flagText = Resources.__("mainFlag" + flag + "NoHTML");

        enabled = createCheckBox(null, true);
        enabled.setToolTipText(HTML.wrap(Resources.__("mainFlagToggleTT", flagText)));
        if (parent != null) {
            enabled.addActionListener(new FlagToggleListener(parent));
        }
        layout(WEST, enabled, 5, WEST, panel);
        layout(NORTH, enabled, 5, NORTH, panel);
        add(enabled);

        label = createLabel(null, null);
        label.setText(HTML.wrap(Resources.__("mainFlagToggle", flagText)));
        label.setToolTipText(enabled.getToolTipText());
        layout(WEST, label, 5, EAST, enabled);
        layout(NORTH, label, 5, NORTH, panel);
        add(label);

        color = createColorPanel(PanelFiles.getFlagColor(flag), 50, 20, 3, "mainFlagColorTT");
        layout(EAST, color.panel, -5, EAST, panel);
        layout(NORTH, color.panel, 5, NORTH, panel);
        add(color.panel);
    }
}
