package edu.uwyo.crawl.ui.frameFlags.panels;

import javax.swing.JButton;

import edu.uwyo.crawl.RestorableView;
import edu.uwyo.crawl.ui.FrameMain;
import edu.uwyo.crawl.ui.panels.PanelFiles;
import net.javamon.dev.utils.JLabelHTML;
import net.javamon.dev.utils.layout.AbstractPanel;

/**
 * The main panel for {@link edu.uwyo.crawl.ui.frameFlags.FrameFlags}.
 * @version 15 February, 2019
 * @author Christopher Stanton
 */
public class PanelMain extends AbstractPanel implements RestorableView {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public PanelFlag[] flags;
    public JButton trim;

    private JLabelHTML instructions;
    private final FrameMain parent;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link PanelMain}. */
    public PanelMain(FrameMain parent) {
        super();
        this.parent = parent;

        setInstructions();

        layoutGrid.start(1, 0, 5, panel, instructions);
        setFlags(parent);
        layoutGrid.end();

        setTrim(parent);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void setEnabled(boolean enabled) {
        for (int i = 0; i < flags.length; i++) {
            flags[i].setEnabled(enabled);
        }
    }

    @Override
    public void validate() {
        boolean atLeastOne = false;
        for (int i = 0; i < flags.length; i++) {
            flags[i].validate();
            if (!flags[i].enabled.isSelected()) {
                atLeastOne = true;
            }
        }
        trim.setEnabled(atLeastOne && parent.main().list.tree.getTree() != null);
    }

    @Override
    public void restoreView() {
        for (int i = 0; i < flags.length; i++) {
            flags[i].restoreView();
        }
    }

    @Override
    public void saveView() {
        for (int i = 0; i < flags.length; i++) {
            flags[i].saveView();
        }
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Sets the instructions text. */
    private void setInstructions() {
        instructions = createLabel("mainFlagInstructions", null);
        layout(WEST, instructions, 5, WEST, panel);
        layout(NORTH, instructions, 5, NORTH, panel);
        layout(EAST, instructions, -5, EAST, panel);
        add(instructions);
    }

    /** Sets the flag panels. */
    private void setFlags(FrameMain parent) {
        flags = new PanelFlag[PanelFiles.getNumFlags()];
        for (int i = 0; i < flags.length; i++) {
            flags[i] = new PanelFlag(parent, i);
            layoutGrid.place(flags[i].panel);
        }
    }

    /** Sets the trim button. */
    private void setTrim(FrameMain parent) {
        trim = createButton("mainFlagTrim", "mainFlagTrimTT", -1, -1);
        trim.addActionListener(new TrimListener(parent));
        layout(WEST, trim, 5, WEST, panel);
        layout(SOUTH, trim, -5, SOUTH, panel);
        add(trim);
    }
}
