package edu.uwyo.crawl.ui.frameFlags.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JOptionPane;

import edu.uwyo.Main;
import edu.uwyo.crawl.ui.FrameMain;
import edu.uwyo.crawl.ui.panels.PanelFiles;
import net.javamon.dev.Console;
import net.javamon.dev.Prompt;
import net.javamon.dev.Resources;

/**
 * A listener used for trimming the current search results to exclude all hidden entries.
 * @version 28 March, 2019
 * @author Christopher Stanton
 */
public class TrimListener implements ActionListener {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private final FrameMain parent;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link TrimListener}. */
    protected TrimListener(FrameMain parent) {
        this.parent = parent;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void actionPerformed(ActionEvent e) {
        int choice = Prompt.show(Resources.__("mainFlagTrimConfirmH"),
                Resources.__("mainFlagTrimConfirmMsg"));

        if (choice == JOptionPane.YES_OPTION) {
            parent.hide();
            Console.windowLoad.start(null, null);
            new Thread(new Runnable() {
                @Override
                public void run() {
                    String oldRoot = Main.database().results().resultTree().getFilePath();
                    String[] paths = Main.database().results().resultTree().getPaths();
                    long[] sizes = Main.database().results().resultTree().getPathSizes();
                    Main.database().results().resultTree().clear(oldRoot);

                    for (int i = 0; i < paths.length; i++) {
                        int flag = Math.max(0, PanelFiles.NODE_FACTORY.getFlagForPath(paths[i]));
                        if (parent.flags().main().flags[flag].enabled.isSelected()) {
                            Main.database().results().resultTree().add(null, paths[i], false,
                                    sizes[i]);
                        }
                    }

                    Console.windowLoad.stop();
                    Main.frameMain.modify();
                    Main.frameMain.show(Main.currSnapshot());
                }
            }).start();
        }
    }
}
