package edu.uwyo.crawl.ui.frameJoin.panels;

import javax.swing.JButton;

import net.javamon.dev.Resources;
import net.javamon.dev.utils.JScrollList;
import net.javamon.dev.utils.layout.AbstractPanel;
import net.javamon.dev.utils.layout.ValidateListSelectionListener;

/**
 * A panel representing the files to be joined.
 * @version 28 March, 2019
 * @author Christopher Stanton
 */
public class PanelList extends AbstractPanel {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public JScrollList list;
    public JButton setMaster;
    public JButton add;
    public JButton remove;

    private int currMasterIndex;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link PanelList}. */
    public PanelList(PanelMain parent) {
        super("mainJoinListBdr", 300, 250);
        currMasterIndex = -1;

        setAdd(parent);
        setRemove(parent);
        setMaster(parent);
        setList(parent);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void setEnabled(boolean enabled) {
        list.setEnabled(enabled);
        setMaster.setEnabled(enabled);
        add.setEnabled(enabled);
        remove.setEnabled(enabled);
    }

    @Override
    public void validate() {
        final int index = list.list().getSelectedIndex();
        setMaster.setEnabled(index != -1 && index != currMasterIndex);
        remove.setEnabled(index != -1 && list.model().getSize() > 0);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** @return the array of String entries in the list */
    protected String[] getEntries() {
        String[] toReturn = new String[list.model().getSize()];
        for (int i = 0; i < toReturn.length; i++) {
            toReturn[i] = list.model().getElementAt(i);
            if (i == currMasterIndex) {
                toReturn[i] = toReturn[i].substring(0, toReturn[i].length() - 1);
            }
        }
        return toReturn;
    }

    /** @return the integer index of the current master file, or -1 if none */
    protected int getMasterIndex() {
        return currMasterIndex;
    }

    /** Sets the master index to the currently selected entry. */
    protected void setMasterIndex() {
        currMasterIndex = list.list().getSelectedIndex();
        String entry = list.model().getElementAt(currMasterIndex);
        list.model().setElementAt(Resources.__("mainJoinListMasterFile", entry), currMasterIndex);
    }

    /** Removes the current master index, if any. */
    protected void removeMasterIndex() {
        if (currMasterIndex != -1) {
            String entry = list.model().getElementAt(currMasterIndex);
            list.model().setElementAt(entry.substring(0, entry.length() - 1), currMasterIndex);
        }
        currMasterIndex = -1;
    }

    /** Sets the add button. */
    private void setAdd(PanelMain parent) {
        add = createButton("mainJoinListAdd", "mainJoinListAddTT", -1, -1);
        add.addActionListener(new AddListener(parent));
        layout(WEST, add, 5, WEST, panel);
        layout(NORTH, add, 5, NORTH, panel);
        add(add);
    }

    /** Sets the remove button. */
    private void setRemove(PanelMain parent) {
        remove = createButton("mainJoinListRemove", "mainJoinListRemoveTT", -1, -1);
        remove.addActionListener(new RemoveListener(parent));
        layout(WEST, remove, 5, EAST, add);
        layout(NORTH, remove, 5, NORTH, panel);
        add(remove);
    }

    /** Sets the set-master button. */
    private void setMaster(PanelMain parent) {
        setMaster = createButton("mainJoinListMaster", "mainJoinListMasterTT", -1, -1);
        setMaster.addActionListener(new SetMasterListener(parent));
        layout(WEST, setMaster, 25, EAST, remove);
        layout(NORTH, setMaster, 5, NORTH, panel);
        add(setMaster);
    }

    /** Sets the file list. */
    private void setList(PanelMain parent) {
        list = createList("mainJoinListItemsTT", JScrollList.SINGLE, JScrollList.VERTICAL,
                100, 100, null);
        list.list().addListSelectionListener(new ValidateListSelectionListener(parent));
        layoutFill(list.panel, 5, add, panel, panel, panel);
        add(list.panel);
    }
}
