package edu.uwyo.crawl.ui.frameJoin.panels;

import javax.swing.JButton;

import net.javamon.dev.utils.JLabelHTML;
import net.javamon.dev.utils.layout.AbstractPanel;

/**
 * The main panel for {@link edu.uwyo.crawl.ui.frameJoin.FrameJoin}.
 * @version 28 March, 2019
 * @author Christopher Stanton
 */
public class PanelMain extends AbstractPanel {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public PanelFlags flags;
    public PanelList list;
    public PanelMode mode;
    public JButton join;

    private JLabelHTML instructions;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link PanelMain}. */
    public PanelMain() {
        super();
        setInstructions();
        setMode();
        setFlags();
        setJoin();
        setList();
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void setEnabled(boolean enabled) {
        list.setEnabled(enabled);
        mode.setEnabled(enabled);
        flags.setEnabled(enabled);
        join.setEnabled(enabled);
    }

    @Override
    public void validate() {
        list.validate();
        mode.validate();
        flags.validate();
        join.setEnabled(list.list.model().getSize() > 1 && list.getMasterIndex() != -1);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Sets the instructions text. */
    private void setInstructions() {
        instructions = createLabel("mainJoinInstructions", null);
        layout(WEST, instructions, 5, WEST, panel);
        layout(NORTH, instructions, 5, NORTH, panel);
        layout(EAST, instructions, -5, EAST, panel);
        add(instructions);
    }

    /** Sets the mode panel. */
    private void setMode() {
        mode = new PanelMode(this);
        layout(EAST, mode.panel, -5, EAST, panel);
        layout(NORTH, mode.panel, 5, SOUTH, instructions);
        add(mode.panel);
    }

    /** Sets the flag panel. */
    private void setFlags() {
        flags = new PanelFlags();
        layout(EAST, flags.panel, -5, EAST, panel);
        layout(NORTH, flags.panel, 5, SOUTH, mode.panel);
        add(flags.panel);
    }

    /** Sets the join button. */
    private void setJoin() {
        join = createButton("mainJoin", "mainJoinTT", -1, -1);
        join.addActionListener(new JoinListener(this));
        layout(EAST, join, -5, EAST, panel);
        layout(SOUTH, join, -5, SOUTH, panel);
        add(join);
    }

    /** Sets the list panel. */
    private void setList() {
        list = new PanelList(this);
        layoutFill(list.panel, 5, instructions, mode.panel, panel, panel);
        add(list.panel);
    }
}
