package edu.uwyo.crawl.ui.frameJoin.panels;

import javax.swing.JRadioButton;

import net.javamon.dev.utils.layout.AbstractPanel;
import net.javamon.dev.utils.layout.ValidateListener;

/**
 * A panel representing the type of join operation to use.
 * @version 28 March, 2019
 * @author Christopher Stanton
 */
public class PanelMode extends AbstractPanel {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public JRadioButton union;
    public JRadioButton intersect;
    public JRadioButton nonIntersect;
    public JRadioButton totalIntersect;
    public JRadioButton totalDifference;
    public JRadioButton symmetricDifference;
    public JRadioButton asymmetricDifference;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link PanelMode}. */
    public PanelMode(PanelMain parent) {
        super("mainJoinModeBdr", 260, 125);

        startButtonGroup();
        setUnion(parent);

        layoutGrid.start(0, 3, 0, panel, union);
        setIntersect(parent);
        setTotalIntersect(parent);
        setSymmetricDifference(parent);
        setNonIntersect(parent);
        setTotalDifference(parent);
        setAsymmetricDifference(parent);
        layoutGrid.end();

        endButtonGroup();
        union.setSelected(true);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void setEnabled(boolean enabled) {
        union.setEnabled(enabled);
        intersect.setEnabled(enabled);
        nonIntersect.setEnabled(enabled);
        totalIntersect.setEnabled(enabled);
        totalDifference.setEnabled(enabled);
        symmetricDifference.setEnabled(enabled);
        asymmetricDifference.setEnabled(enabled);
    }

    @Override
    public void validate() {}

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Sets the union button. */
    private void setUnion(PanelMain parent) {
        union = createRadioButton("mainJoinModeUnion", "mainJoinModeUnionTT", 120, 25, true);
        union.addActionListener(new ValidateListener(parent));
        layout(WEST, union, 0, WEST, panel);
        layout(NORTH, union, 0, NORTH, panel);
        add(union);
    }

    /** Sets the intersect button. */
    private void setIntersect(PanelMain parent) {
        intersect = createRadioButton("mainJoinModeIntersect", "mainJoinModeIntersectTT", 120, 25,
                true);
        intersect.addActionListener(new ValidateListener(parent));
        layoutGrid.place(intersect);
    }

    /** Sets the total intersect button. */
    private void setTotalIntersect(PanelMain parent) {
        totalIntersect = createRadioButton("mainJoinModeTotalIntersect",
                "mainJoinModeTotalIntersectTT", 120, 25, true);
        totalIntersect.addActionListener(new ValidateListener(parent));
        layoutGrid.place(totalIntersect);
    }

    /** Sets the symmetric difference button. */
    private void setSymmetricDifference(PanelMain parent) {
        symmetricDifference = createRadioButton("mainJoinModeSymmetricDifference",
                "mainJoinModeSymmetricDifferenceTT", 120, 25, true);
        symmetricDifference.addActionListener(new ValidateListener(parent));
        layoutGrid.place(symmetricDifference);
    }

    /** Sets the non-intersect button. */
    private void setNonIntersect(PanelMain parent) {
        nonIntersect = createRadioButton("mainJoinModeNonIntersect",
                "mainJoinModeNonIntersectTT", 120, 25, true);
        nonIntersect.addActionListener(new ValidateListener(parent));
        layoutGrid.place(nonIntersect);
    }

    /** Sets the total difference button. */
    private void setTotalDifference(PanelMain parent) {
        totalDifference = createRadioButton("mainJoinModeTotalDifference",
                "mainJoinModeTotalDifferenceTT", 120, 25, true);
        totalDifference.addActionListener(new ValidateListener(parent));
        layoutGrid.place(totalDifference);
    }

    /** Sets the asymmetric difference button. */
    private void setAsymmetricDifference(PanelMain parent) {
        asymmetricDifference = createRadioButton("mainJoinModeAsymmetricDifference",
                "mainJoinModeAsymmetricDifferenceTT", 120, 25, true);
        asymmetricDifference.addActionListener(new ValidateListener(parent));
        layoutGrid.place(asymmetricDifference);
    }
}
