package edu.uwyo.crawl.ui.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JFileChooser;

import net.javamon.dev.utils.RelativeFileChooser;

/**
 * A listener for choosing which folder should be searched.
 * @version 8 February, 2019
 * @author Christopher Stanton
 */
public class ChooseFolderListener implements ActionListener {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private final PanelOptions parent;
    private final RelativeFileChooser fileChooser;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link ChooseFolderListener}. */
    protected ChooseFolderListener(PanelOptions parent, RelativeFileChooser fileChooser) {
        this.parent = parent;
        this.fileChooser = fileChooser;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void actionPerformed(ActionEvent e) {
        if (parent.recurse.getSelectedIndex() == PanelOptions.RECURSE_FILE) {
            fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        } else {
            fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        }

        if (parent.folder.getText().length() > 0) {
            fileChooser.setCurrentDirectory(new File(parent.folder.getText()));
        }

        int choice = fileChooser.showOpenDialog(parent.panel);
        if (choice == JFileChooser.APPROVE_OPTION) {
            parent.folder.setText(fileChooser.getSelectedFile().getAbsolutePath());
            parent.folder.getCaret().setDot(parent.folder.getText().length());
            parent.validate();
        }
    }
}
