package edu.uwyo.crawl.ui.panels;

import edu.uwyo.crawl.RestorableView;
import edu.uwyo.crawl.ui.FrameMain;
import net.javamon.dev.utils.layout.AbstractPanel;

/**
 * The main panel for {@link FrameMain}.
 * @version 8 February, 2019
 * @author Christopher Stanton
 */
public class PanelMain extends AbstractPanel implements RestorableView {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public PanelOptions options;
    public PanelOptionsFiles files;
    public PanelOptionsTime time;
    public PanelOptionsSize size;
    public PanelFiles list;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link PanelMain}. */
    public PanelMain() {
        super();
        setOptions();
        setFiles();
        setTime();
        setSize();
        setList();
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void setEnabled(boolean enabled) {
        options.setEnabled(enabled);
        files.setEnabled(enabled);
        time.setEnabled(enabled);
        size.setEnabled(enabled);
        list.setEnabled(enabled);
    }

    @Override
    public void validate() {
        files.validate();
        time.validate();
        size.validate();
        options.validate();
        list.validate();
    }

    @Override
    public void restoreView() {
        options.restoreView();
        files.restoreView();
        time.restoreView();
        size.restoreView();
        list.restoreView();
    }

    @Override
    public void saveView() {
        options.saveView();
        files.saveView();
        time.saveView();
        size.saveView();
        list.saveView();
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Sets the options panel. */
    private void setOptions() {
        options = new PanelOptions();
        layout(WEST, options.panel, 5, WEST, panel);
        layout(NORTH, options.panel, 5, NORTH, panel);
        layout(EAST, options.panel, -5, EAST, panel);
        add(options.panel);
    }

    /** Sets the files panel. */
    private void setFiles() {
        files = new PanelOptionsFiles(this);
        layout(EAST, files.panel, -5, EAST, panel);
        layout(NORTH, files.panel, 5, SOUTH, options.panel);
        add(files.panel);
    }

    /** Sets the time panel. */
    private void setTime() {
        time = new PanelOptionsTime(this);
        layout(EAST, time.panel, -5, EAST, panel);
        layout(NORTH, time.panel, 5, SOUTH, files.panel);
        add(time.panel);
    }

    /** Sets the size panel. */
    private void setSize() {
        size = new PanelOptionsSize(this);
        layout(EAST, size.panel, -5, EAST, panel);
        layout(NORTH, size.panel, 5, SOUTH, time.panel);
        add(size.panel);
    }

    /** Sets the list panel. */
    private void setList() {
        list = new PanelFiles();
        layoutFill(list.panel, 5, options.panel, files.panel, panel, panel);
        add(list.panel);
    }
}
