package edu.uwyo.crawl.ui.panels;

import edu.uwyo.Main;
import edu.uwyo.crawl.RestorableView;
import net.javamon.dev.utils.JScrollPaneText;
import net.javamon.dev.utils.layout.AbstractPanel;
import net.javamon.dev.utils.layout.ValidateFocusListener;
import net.javamon.dev.utils.layout.ValidateKeyListener;
import net.javamon.dev.utils.textPane.JTextPaneMarkup;

/**
 * The options panel for searchable file extensions.
 * @version 8 February, 2019
 * @author Christopher Stanton
 */
public class PanelOptionsFiles extends AbstractPanel implements RestorableView {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                   CONSTANTS                     ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public static final String DELIMITERS = "[ \t\n,]+";
    public static final String ALL_FILES = "*";

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public JScrollPaneText fileTypes;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link PanelOptionsFiles}. */
    public PanelOptionsFiles(PanelMain parent) {
        super("mainPanelOptionsFilesBdr", 230, 75);
        setTypes(parent);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void setEnabled(boolean enabled) {
        fileTypes.setEnabled(enabled);
    }

    @Override
    public void validate() {
        String text = fileTypes.getText();
        if (text.length() == 0) {
            fileTypes.setText(ALL_FILES);
            return;
        }

        String[] types = text.split(DELIMITERS);
        StringBuilder validated = new StringBuilder();
        for (int i = 0; i < types.length; i++) {
            String validatedType;
            if (types[i].contains(ALL_FILES)) {
                fileTypes.setText(ALL_FILES);
                return;
            } else {
                validatedType = "." + types[i].replace(".", "");
            }
            validated.append(validatedType + " ");
        }
        fileTypes.setText(validated.toString());
    }

    @Override
    public void restoreView() {
        String lastFiles = Main.readSetting("lastFileTypes");
        if (lastFiles != null) {
            fileTypes.setText(lastFiles);
        }
    }

    @Override
    public void saveView() {
        Main.writeSetting("lastFileTypes", fileTypes.getText());
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** @return an array of String valid types, or {@code null} if all types are allowed */
    public String[] getValidTypes() {
        String text = fileTypes.getText();
        if (text.equals(ALL_FILES)) {
            return null;
        }
        return text.split(DELIMITERS);
    }

    /** Sets the types field. */
    private void setTypes(PanelMain parent) {
        fileTypes = createScrollText(JTextPaneMarkup.MARKUP_NONE, null,
                "mainPanelOptionsFilesTT", 4, 5, true, true, false, true,
                borderWidth() - left() - right(),
                borderHeight() - top() - bottom());
        fileTypes.setText(ALL_FILES);
        fileTypes.getPane().addFocusListener(new ValidateFocusListener(parent));
        fileTypes.getPane().addKeyListener(new ValidateKeyListener(parent));
        layoutFill(fileTypes, 5, panel, panel, panel, panel);
        add(fileTypes);
    }
}
