package edu.uwyo.crawl.ui.panels;

import javax.swing.JTextField;

import edu.uwyo.Main;
import edu.uwyo.crawl.RestorableView;
import net.javamon.dev.Resources;
import net.javamon.dev.utils.HTML;
import net.javamon.dev.utils.JLabelHTML;
import net.javamon.dev.utils.comboBox.ExtendableJComboBox;
import net.javamon.dev.utils.layout.AbstractPanel;
import net.javamon.dev.utils.layout.Validate;
import net.javamon.dev.utils.layout.ValidateFocusListener;
import net.javamon.dev.utils.layout.ValidateKeyListener;
import net.javamon.dev.utils.layout.ValidateListener;

/**
 * The options panel for determining file size constraints.
 * @version 8 February, 2019
 * @author Christopher Stanton
 */
public class PanelOptionsSize extends AbstractPanel implements RestorableView {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                   CONSTANTS                     ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public static final int NONE = 0;
    public static final int SMALLER = 1;
    public static final int LARGER = 2;
    public static final int BETWEEN = 3;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public ExtendableJComboBox<String> mode;
    public JTextField warning;
    public ExtendableJComboBox<String> warningUnits;
    public JTextField error;
    public ExtendableJComboBox<String> errorUnits;

    private JLabelHTML warningLabel;
    private JLabelHTML errorLabel;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Creates a new {@link PanelOptionsSize} */
    public PanelOptionsSize(PanelMain parent) {
        super("mainPanelOptionsSizeBdr", 230, 118);
        setMode(parent);
        setWarning(parent);
        setError(parent);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void setEnabled(boolean enabled) {
        mode.setEnabled(enabled);
        warning.setEnabled(enabled);
        warningUnits.setEnabled(enabled);
        error.setEnabled(enabled);
        errorUnits.setEnabled(enabled);
    }

    @Override
    public void validate() {
        if (!mode.isEnabled()) {
            mode.setSelectedIndex(0);
            warning.setText("");
            error.setText("");
            warningUnits.setSelectedIndex(0);
            errorUnits.setSelectedIndex(0);
            return;
        }

        warning.setEnabled(mode.getSelectedIndex() != NONE);
        error.setEnabled(mode.getSelectedIndex() != NONE);
        warningUnits.setEnabled(mode.getSelectedIndex() != NONE);
        errorUnits.setEnabled(mode.getSelectedIndex() != NONE);
        if (mode.getSelectedIndex() == NONE) {
            warning.setText("");
            error.setText("");
            warningUnits.setSelectedIndex(0);
            errorUnits.setSelectedIndex(0);
        }

        Validate.textConstrainInt(warning, 1, false);
        Validate.textConstrainInt(error, 1, false);

        final int warningNum = Integer.parseInt(warning.getText());
        final int warningSize = warningUnits.getSelectedIndex();
        final int errorNum = Integer.parseInt(error.getText());
        final int errorSize = errorUnits.getSelectedIndex();

        int warningAmount = getAmount(warningNum, warningSize);
        int errorAmount = getAmount(errorNum, errorSize);

        if (mode.getSelectedIndex() == LARGER && errorAmount <= warningAmount) {
            error.setText((warningNum + 1) + "");
            errorUnits.setSelectedIndex(warningSize);

        } else if (mode.getSelectedIndex() == SMALLER && errorAmount >= warningAmount) {
            if (warningNum > 1) {
                error.setText((warningNum - 1) + "");
            } else if (warningSize > 0) {
                error.setText("1023");
                errorUnits.setSelectedIndex(warningSize - 1);
            } else {
                warning.setText((errorNum + 1) + "");
                warningUnits.setSelectedIndex(errorSize);
            }

        } else if (mode.getSelectedIndex() == BETWEEN && errorAmount < warningAmount) {
            error.setText((warningNum + 1) + "");
            errorUnits.setSelectedIndex(warningSize);
        }

        if (mode.getSelectedIndex() == BETWEEN) {
            warningLabel.setText(HTML.wrap(Resources.__("mainPanelOptionsSizeMin")));
            warningLabel.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeMinTT")));
            warningUnits.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeMinTT")));
            warning.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeMinTT")));
            errorLabel.setText(HTML.wrap(Resources.__("mainPanelOptionsSizeMax")));
            errorLabel.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeMaxTT")));
            errorUnits.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeMaxTT")));
            error.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeMaxTT")));
        } else {
            warningLabel.setText(HTML.wrap(Resources.__("mainPanelOptionsSizeWarning")));
            warningLabel.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeWarningTT")));
            warningUnits.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeWarningTT")));
            warning.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeWarningTT")));
            errorLabel.setText(HTML.wrap(Resources.__("mainPanelOptionsSizeError")));
            errorLabel.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeErrorTT")));
            errorUnits.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeErrorTT")));
            error.setToolTipText(HTML.wrap(Resources.__("mainPanelOptionsSizeErrorTT")));
        }
    }

    @Override
    public void restoreView() {
        String lastMode = Main.readSetting("lastSizeMode");
        String lastWarning = Main.readSetting("lastSizeWarning");
        String lastWarningUnits = Main.readSetting("lastSizeWarningUnits");
        String lastError = Main.readSetting("lastSizeError");
        String lastErrorUnits = Main.readSetting("lastSizeErrorUnits");

        if (lastMode != null) {
            mode.setSelectedIndex(Integer.parseInt(lastMode));
        }
        if (lastWarning != null) {
            warning.setText(lastWarning);
        }
        if (lastWarningUnits != null) {
            warningUnits.setSelectedIndex(Integer.parseInt(lastWarningUnits));
        }
        if (lastError != null) {
            error.setText(lastError);
        }
        if (lastErrorUnits != null) {
            errorUnits.setSelectedIndex(Integer.parseInt(lastErrorUnits));
        }
    }

    @Override
    public void saveView() {
        Main.writeSetting("lastSizeMode", mode.getSelectedIndex() + "");
        Main.writeSetting("lastSizeWarning", warning.getText());
        Main.writeSetting("lastSizeWarningUnits", warningUnits.getSelectedIndex() + "");
        Main.writeSetting("lastSizeError", error.getText());
        Main.writeSetting("lastSizeErrorUnits", errorUnits.getSelectedIndex() + "");
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** @return the integer current warning amount size, in bytes */
    public long getWarningBytes() {
        return getAmount(Integer.parseInt(warning.getText()), warningUnits.getSelectedIndex())
                * 1024L;
    }

    /** @return the integer current error amount size, in bytes */
    public long getErrorBytes() {
        return getAmount(Integer.parseInt(error.getText()), errorUnits.getSelectedIndex())
                * 1024L;
    }

    /** @return the integer {@code amount}, converted to KB */
    private int getAmount(int amount, int unitIndex) {
        switch (unitIndex) {
            case 1:     return amount * 1024;
            case 2:     return amount * 1024 * 1024;
            default:    return amount;
        }
    }

    /** Sets the size mode chooser. */
    private void setMode(PanelMain parent) {
        JLabelHTML label = createLabel("mainPanelOptionsSizeMode", "mainPanelOptionsSizeModeTT");
        layout(WEST, label, 5, WEST, panel);
        layout(NORTH, label, 5, NORTH, panel);
        add(label);

        mode = createComboBox("mainPanelOptionsSizeModeTT",
                Resources.__("mainPanelOptionsSizeMode0"),
                Resources.__("mainPanelOptionsSizeMode1"),
                Resources.__("mainPanelOptionsSizeMode2"),
                Resources.__("mainPanelOptionsSizeMode3"));
        mode.addActionListener(new ValidateListener(parent));
        layout(EAST, mode, -5, EAST, panel);
        layout(NORTH, mode, 5, NORTH, panel);
        add(mode);
    }

    /** Sets the warning field and units. */
    private void setWarning(PanelMain parent) {
        warningLabel = createLabel("mainPanelOptionsSizeWarning",
                "mainPanelOptionsSizeWarningTT", 100);
        layout(WEST, warningLabel, 5, WEST, panel);
        layout(NORTH, warningLabel, 5, SOUTH, mode);
        add(warningLabel);

        warningUnits = createUnits(parent, "mainPanelOptionsSizeWarningTT");
        layout(EAST, warningUnits, -5, EAST, panel);
        layout(NORTH, warningUnits, 5, SOUTH, mode);
        add(warningUnits);

        warning = createTextField(null, 5, "mainPanelOptionsSizeWarningTT");
        warning.addFocusListener(new ValidateFocusListener(parent));
        warning.addKeyListener(new ValidateKeyListener(parent));
        layout(EAST, warning, -5, WEST, warningUnits);
        layout(NORTH, warning, 5, SOUTH, mode);
        add(warning);
    }

    /** Sets the error field and units. */
    private void setError(PanelMain parent) {
        errorLabel = createLabel("mainPanelOptionsSizeError",
                "mainPanelOptionsSizeErrorTT", 100);
        layout(WEST, errorLabel, 5, WEST, panel);
        layout(NORTH, errorLabel, 5, SOUTH, warningUnits);
        add(errorLabel);

        errorUnits = createUnits(parent, "mainPanelOptionsSizeErrorTT");
        layout(EAST, errorUnits, -5, EAST, panel);
        layout(NORTH, errorUnits, 5, SOUTH, warningUnits);
        add(errorUnits);

        error = createTextField(null, 5, "mainPanelOptionsSizeErrorTT");
        error.addFocusListener(new ValidateFocusListener(parent));
        error.addKeyListener(new ValidateKeyListener(parent));
        layout(EAST, error, -5, WEST, errorUnits);
        layout(NORTH, error, 5, SOUTH, warningUnits);
        add(error);
    }

    /** @return a new {@link ExtendableJComboBox} that has been populated with file size units */
    private ExtendableJComboBox<String> createUnits(PanelMain parent, String tooltipKey) {
        ExtendableJComboBox<String> units = createComboBox(tooltipKey,
                Resources.__("mainPanelOptionsSizeKB"),
                Resources.__("mainPanelOptionsSizeMB"),
                Resources.__("mainPanelOptionsSizeGB"));
        units.addActionListener(new ValidateListener(parent));
        return units;
    }
}
