package edu.uwyo.crawl.ui.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import edu.uwyo.Main;
import edu.uwyo.crawl.ui.tree.FileDisplayTree;
import net.javamon.dev.Console;
import net.javamon.dev.Prompt;
import net.javamon.dev.Resources;
import net.javamon.dev.windowLoad.WindowLoad.WindowLoadInvokable;

/**
 * A listener for beginning the search process.
 * @version 8 February, 2019
 * @author Christopher Stanton
 */
public class ProcessListener implements ActionListener {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                   CONSTANTS                     ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private static final WindowLoadInvokable INVOKABLE = new WindowLoadInvokable() {
        @Override
        public String getText() {
            return Resources.__("mainNumSearched", Main.database().numSearched());
        }
    };

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void actionPerformed(ActionEvent e) {
        if (Main.frameMain.main().list.tree.getTree() != null
                && Main.frameMain.main().list.tree.getTree().isFlat()
                && Main.frameMain.main().options.duplicate.getSelectedIndex() > 0
                && Main.frameMain.main().options.recurse.getSelectedIndex() == PanelOptions.RECURSE_RESULTS) {
            Prompt.error(Resources.__("mainPanelFileMenuDuplicateErrorH"),
                    Resources.__("mainPanelFileMenuDuplicateErrorMsg"));
            return;
        }

        Main.query().update();
        Main.frameMain.saveView();
        Main.frameMain.hide();

        Main.allowWindowKill(true);
        if (Main.frameMain.main().options.recurse.getSelectedIndex() == PanelOptions.RECURSE_FILE) {
            Console.windowLoad.start(null, null); //no need for file count
        } else {
            Console.windowLoad.start(null, INVOKABLE);
        }

        new Thread(new Runnable() {
            @Override
            public void run() {
                FileDisplayTree resultTree = Main.query().run(Main.database(), false);
                Main.frameMain.main().list.updateView(resultTree);

                Console.windowLoad.stop();
                Main.frameMain.modify();
                Main.frameMain.show(Main.currSnapshot());
                Main.allowWindowKill(false);
            }
        }).start();
    }
}
