package edu.uwyo.crawl.ui.popup;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import edu.uwyo.Main;
import edu.uwyo.crawl.ui.panels.PanelFiles;

/**
 * A listener for overriding a file's current status flag.
 * @version 25 February, 2019
 * @author Christopher Stanton
 */
public class ChangeFlagListener implements ActionListener {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private final FilePopupMenu menu;
    private final int flag;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Creates a new {@link ChangeFlagListener}.
     * @param menu the parent {@link FilePopupMenu}
     * @param flag the integer status flag to be set by this listener
     */
    protected ChangeFlagListener(FilePopupMenu menu, int flag) {
        this.menu = menu;
        this.flag = flag;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void actionPerformed(ActionEvent e) {
        Main.frameMain.modify();

        int setCount = 0;
        int length = 0; //currNodes().length might include folders, which can't be flagged
        for (int i = 0; i < menu.currNodes().length; i++) {
            if (menu.currNodes()[i].getFilePath() != null) {
                length++;
                if (flag == PanelFiles.indexOf(menu.currNodes()[i].getColor())) {
                    setCount++;
                }
            }
        }

        final boolean set;
        if (length == 1) {
            set = (setCount == 0);
        } else if (setCount == 0) {
            set = true;
        } else if (setCount == length) {
            set = false;
        } else {
            set = setCount > (length / 2);
        }

        boolean requireRedraw = false;
        for (int i = 0; i < menu.currNodes().length; i++) {
            if (menu.currNodes()[i].getFilePath() != null) {
                if (set) {
                    menu.currNodes()[i].setColor(PanelFiles.getFlagColor(flag));
                    if (!Main.frameMain.flags().main().flags[flag].enabled.isSelected()) {
                        requireRedraw = true;
                    }
                } else {
                    menu.currNodes()[i].setColor(null);
                    if (!Main.frameMain.flags().main().flags[PanelFiles.FLAG_OKAY].enabled.isSelected()) {
                        requireRedraw = true;
                    }
                }
            }
        }

        if (requireRedraw) {
            Main.frameMain.main().list.tree.redraw(); //removes flagged nodes if flag was filtered
        }
    }
}
