package edu.uwyo.crawl.ui.popup;

import java.awt.Color;

import edu.uwyo.Main;
import edu.uwyo.crawl.Query;
import edu.uwyo.crawl.ui.panels.PanelFiles;
import edu.uwyo.crawl.ui.tree.AbstractFileNode;

/**
 * A file node in which colors are obtained from the current user profile.
 * Users can set override colors via {@link #setColor(Color)}.
 * @version 21 February, 2019
 * @author Christopher Stanton
 */
public class ColorHashFileNode extends AbstractFileNode {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                   CONSTANTS                     ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private static final long serialVersionUID = -3368228857693840499L;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Creates a new {@link ColorHashFileNode}.
     * @param path the String absolute file path to be associated with this node
     * @param contents the {@link Object} contents to be placed in this node;
     *  in most cases, this is the String to be displayed by the node
     */
    public ColorHashFileNode(String path, Object contents) {
        super(path, contents);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public Color getColor() {
        return PanelFiles.getFlagColor(getFlagForPath(getFilePath()));
    }

    @Override
    public void setColor(Color color) {
        if (getFilePath() != null) {
            if (color == null) {
                Main.removeFlag(getFilePath());
                return;
            }

            byte chosenFlag = (byte) PanelFiles.indexOf(color);
            if (chosenFlag != -1) {
                Main.setFlag(getFilePath(), chosenFlag);
            }
        }
    }

    @Override
    public ColorHashFileNode create(String path, Object contents) {
        return new ColorHashFileNode(path, contents);
    }

    @Override
    public boolean excludeFromTree() {
        if (getFilePath() != null) { //never exclude directories
            Color color = getColor();
            int index = (color == null) ? PanelFiles.FLAG_OKAY : PanelFiles.indexOf(color);
            if (index != -1 && !Main.frameMain.flags().main().flags[index].enabled.isSelected()) {
                return true;
            }
        }
        return false;
    }

    @Override
    public int getFlagForPath(String filePath) {
        if (filePath == null) { return -1; }

        int sizeFlag = Main.query().getSizeDescriptor(filePath);
        Byte flag = Main.getFlag(filePath);
        byte setFlag = (flag == null) ? (byte) -1 : flag;

        if (setFlag == -1 && sizeFlag == Query.SIZE_OKAY) {
            return -1;
        }

        if (setFlag == -1) {
            switch (sizeFlag) {
                case Query.SIZE_ERROR:      return PanelFiles.FLAG_ERROR;
                case Query.SIZE_WARNING:    return PanelFiles.FLAG_WARNING;
                default:                    return -1;
            }
        }
        return setFlag;
    }
}
