package edu.uwyo.crawl.ui.popup;

import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import edu.uwyo.crawl.ui.tree.FileDisplayTree;
import net.javamon.dev.ClipboardTools;

/**
 * A listener for exporting the current selection.
 * @version 19 April, 2019
 * @author Christopher Stanton
 */
public class ExportListener implements ActionListener {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                   CONSTANTS                     ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** Indicates that all files and subfolders should be exported. */
    public static final int ALL = 0;

    /** Indicates that the current selection, including subfolders, should be exported. */
    public static final int SUB = 1;

    /** Indicates that the current selection, excluding subfolders, should be exported. */
    public static final int SINGLE = 2;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private final FilePopupMenu menu;
    private final int mode;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Creates a new {@link ExportListener}.
     * @param menu the parent {@link FilePopupMenu}
     * @param mode the integer type of export to perform:
     *  {@link #ALL}, {@link #SUB}, or {@link #SINGLE}
     */
    protected ExportListener(FilePopupMenu menu, int mode) {
        this.menu = menu;
        this.mode = mode;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void actionPerformed(ActionEvent e) {
        StringBuilder builder = new StringBuilder();

        if (mode == ALL) {
            String[] paths = menu.getTree().getPaths();
            for (int i = 0; i < paths.length; i++) {
                builder.append(paths[i] + "\n");
            }

        } else if (mode == SINGLE) {
            for (int i = 0; i < menu.currNodes().length; i++) {
                String nodePath = menu.currNodes()[i].getFilePath();
                if (nodePath != null) {
                    builder.append(new File(nodePath).getAbsoluteFile() + "\n");
                }
            }

        } else {
            for (int i = 0; i < menu.currNodes().length; i++) {
                String nodePath = menu.currNodes()[i].getFilePath();
                File f = menu.currNodes()[i].getNodePathAsFile(menu.isFlat());

                if (nodePath != null) {
                    builder.append(new File(nodePath).getAbsoluteFile() + "\n");
                } else if (f != null) {
                    FileDisplayTree tree = menu.getTree().getTree(menu.currNodes()[i].getPath());
                    String[] paths = tree.getPaths();
                    for (int j = 0; j < paths.length; j++) {
                        builder.append(paths[j] + "\n");
                    }
                }
            }
        }

        ClipboardTools.store(new StringSelection(builder.toString()));
    }
}
