package edu.uwyo.crawl.ui.tree;

import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

/**
 * An abstract class representing a context menu to be displayed when one or more nodes
 *  have been selected.
 * <p>
 * <i>Only one instance of this class should be instantiated
 *  and passed into {@link PanelFileTree}.</i>
 * That instance will then be automatically displayed for any node in response to a context event.
 * @version 21 February, 2019
 * @author Christopher Stanton
 */
public abstract class AbstractFileNodePopup {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private FileDisplayTree resultTree;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                ABSTRACT METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Shows this popup for the given selected node(s).
     * @param nodes the array of {@link AbstractFileNode} that are currently selected
     * @param flat a boolean, {@code true} if the current tree is "flat"
     *  (i.e. not a hierarchical file view)
     * @param e the {@link MouseEvent} that triggered this popup
     */
    protected abstract void show(AbstractFileNode[] nodes, boolean flat, MouseEvent e);

    /**
     * Presses an accelerator key for the given selected node(s).
     * Will only be called if the popup menu is not visible.
     * Because this method will be called for <u>all</u> key presses
     *  (not just for keys registered in this menu), {@code false} should be returned
     *  for any unregistered keys.
     * @param nodes the array of {@link AbstractFileNode} that are currently selected
     * @param e the {@link KeyEvent} that triggered this popup
     * @return a boolean, {@code true} if the key press was processed
     */
    protected abstract boolean pressKey(AbstractFileNode[] nodes, KeyEvent e);

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /** @return the {@link FileDisplayTree} currently assigned to this node's parent */
    public FileDisplayTree getTree() {
        return resultTree;
    }

    /**
     * Sets the file tree that should be used for obtaining entries in the tree.
     * @param resultTree the {@link FileDisplayTree} to be displayed by this tree
     */
    protected void setTree(FileDisplayTree resultTree) {
        this.resultTree = resultTree;
    }
}
