package edu.uwyo.crawl.ui.tree;

import java.awt.event.MouseEvent;

import javax.swing.JTree;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

/**
 * A version of {@link JTree} in which {@link #getToolTipText()} returns the text assigned
 *  to the current {@link AbstractFileNode}, if any.
 * The per-node tooltips will not function until the tree is registered
 *  via {@code ToolTipManager.sharedInstance().registerComponent(tree);}.
 * @version 21 March, 2019
 * @author Christopher Stanton
 */
public class FileNodeToolTipTree extends JTree {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                   CONSTANTS                     ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private static final long serialVersionUID = -6330926524819634605L;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Creates a new {@link FileNodeToolTipTree}.
     * @param root the root {@link TreeNode}
     * @see JTree#JTree(TreeNode)
     */
    public FileNodeToolTipTree(TreeNode root) {
        super(root);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public String getToolTipText(MouseEvent event) {
        int row = getRowForLocation(event.getX(), event.getY());
        if (row == -1) {
            return null;
        }

        TreePath currPath = getPathForRow(row);
        AbstractFileNode node = (AbstractFileNode) currPath.getLastPathComponent();
        return node.getToolTipText();
    }
}
