package edu.uwyo.crawl.ui.tree;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;

/**
 * A custom renderer to be paired with a {@link AbstractFileEntryRenderer}.
 * Allows nodes to be highlighted based on the criteria in a {@link AbstractFileNode} instance.
 * <p>
 * When provided a {@link DefaultFileEntryRenderer}, this renderer behaves in a visually identical
 *  manner to that of a normal {@link FileNodeToolTipTree}.
 * @version 21 February, 2019
 * @author Christopher Stanton
 */
public class FileTreeCellRenderer extends DefaultTreeCellRenderer {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                   CONSTANTS                     ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private static final long serialVersionUID = 504886622455016470L;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private final AbstractFileEntryRenderer renderer;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Creates a new {@link FileTreeCellRenderer}.
     * @param entryRenderer the {@link AbstractFileEntryRenderer} used for rendering
     *  each visible tree node
     */
    protected FileTreeCellRenderer(AbstractFileEntryRenderer entryRenderer) {
        this.renderer = entryRenderer;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected,
            boolean expanded, boolean leaf, int row, boolean focus) {
        super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, focus);

        AbstractFileNode node = (AbstractFileNode) value;
        Color background = node.getColor();
        String entry = node.getUserObject().toString();

        return renderer.set(getIcon(), getIconTextGap(), entry, background, tree.getBackground(),
                selected, getBackgroundSelectionColor());
    }
}
