package edu.uwyo.crawl.ui.tree;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JTree;
import javax.swing.tree.TreePath;

/**
 * A listener used for showing a context menu when a node is clicked.
 * @version 21 February, 2019
 * @author Christopher Stanton
 */
public class FileTreeContextListener extends MouseAdapter {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private final JTree tree;
    private final AbstractFileNodePopup popup;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Creates a new {@link FileTreeContextListener}.
     * @param tree the {@link JTree} that will own this listener
     * @param popup the {@link AbstractFileNodePopup} to display when a mouse event occurs
     */
    protected FileTreeContextListener(JTree tree, AbstractFileNodePopup popup) {
        this.tree = tree;
        this.popup = popup;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void mousePressed(MouseEvent e) {
        showPopup(e);
    }

    @Override
    public void mouseReleased(MouseEvent e) {
        showPopup(e);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    METHODS                      ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Shows the popup.
     * The popup is only shown if platform-dependent popup triggers (e.g. right-click on Windows)
     *  have been satisfied.
     * @param e the {@link MouseEvent} that triggered this listener
     */
    private void showPopup(MouseEvent e) {
        if (popup != null && e.isPopupTrigger()) {
            final int row = tree.getClosestRowForLocation(e.getX(), e.getY());
            final TreePath path = tree.getPathForRow(row);
            final AbstractFileNode node = ((AbstractFileNode) path.getLastPathComponent());

            tree.setSelectionRow(row);
            popup.show(node, e);
        }
    }
}
