package edu.uwyo.crawl.ui.tree;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.tree.TreePath;

/**
 * A listener used for accessing the key accelerator shortcuts of a context menu
 *  for one or more selected node(s).
 * @version 19 March, 2019
 * @author Christopher Stanton
 */
public class FileTreeKeyListener implements KeyListener {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                    FIELDS                       ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private final FileNodeToolTipTree tree;
    private final AbstractFileNodePopup popup;

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //                  CONSTRUCTORS                   ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Creates a new {@link FileTreeKeyListener}.
     * @param tree the {@link FileNodeToolTipTree} that will own this listener
     * @param popup the {@link AbstractFileNodePopup} to display when a key event occurs
     */
    protected FileTreeKeyListener(FileNodeToolTipTree tree, AbstractFileNodePopup popup) {
        this.tree = tree;
        this.popup = popup;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //              OVERRIDDEN METHODS                 ////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void keyTyped(KeyEvent e) {}

    @Override
    public void keyPressed(KeyEvent e) {
        if (popup != null) {
            int[] rows = tree.getSelectionRows();
            if (rows != null && rows.length > 0) {
                AbstractFileNode[] nodes = new AbstractFileNode[rows.length];
                for (int i = 0; i < rows.length; i++) {
                    TreePath path = tree.getPathForRow(rows[i]);
                    nodes[i] = ((AbstractFileNode) path.getLastPathComponent());
                }

                if (popup.pressKey(nodes, e)) {
                    tree.setSelectionRow(-1); //required to refresh selected node's highlighting
                    tree.setSelectionRows(rows);
                }
            }
        }
    }

    @Override
    public void keyReleased(KeyEvent e) {}
}
